/*
 * FILE:
 *   udp_socket.hpp
 *
 * PURPOSE:
 *   Wrapper over UDP sockets. Supports both unicast and broadcast
 *   sending, and receiving.
 *
 * AUTHOR:
 *   Stephen Thompson
 *
 * CREATED:
 *   14-Nov-2008
 *
 * COPYRIGHT:
 *   Copyright (C) Stephen Thompson, 2008 - 2009.
 *
 *   This file is part of the "Coercri" software library. Usage of "Coercri"
 *   is permitted under the terms of the Boost Software License, Version 1.0, 
 *   the text of which is displayed below.
 *
 *   Boost Software License - Version 1.0 - August 17th, 2003
 *
 *   Permission is hereby granted, free of charge, to any person or organization
 *   obtaining a copy of the software and accompanying documentation covered by
 *   this license (the "Software") to use, reproduce, display, distribute,
 *   execute, and transmit the Software, and to prepare derivative works of the
 *   Software, and to permit third-parties to whom the Software is furnished to
 *   do so, all subject to the following:
 *
 *   The copyright notices in the Software and this entire statement, including
 *   the above license grant, this restriction and the following disclaimer,
 *   must be included in all copies of the Software, in whole or in part, and
 *   all derivative works of the Software, unless such copies or derivative
 *   works are solely in the form of machine-executable object code generated by
 *   a source language processor.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 *   SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 *   FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *   DEALINGS IN THE SOFTWARE.
 *
 */

#ifndef COERCRI_UDP_SOCKET_HPP
#define COERCRI_UDP_SOCKET_HPP

#include "boost/noncopyable.hpp"

#include <string>

namespace Coercri {

    class UDPSocket : boost::noncopyable {
    public:
        // a UDPSocket can be bound (in which case it can both send
        // packets, and receive packets on the port it is bound to) or
        // unbound (in which case it can only send). See
        // NetworkDriver::createUDPSocket.
        
        virtual ~UDPSocket() { }

        // NOTE: Msgs are sent/received as strings. It might be better
        // to use vector<unsigned char> instead (for consistency with
        // NetworkConnection), but currently I think it's more
        // convenient to keep them as strings, because I am mostly
        // using the UDPSocket/TCPSocket classes for ASCII rather than
        // binary data at the moment...
        
        // Send a message to a specific address and port.
        virtual void send(const std::string &address, int port, const std::string &msg) = 0;
        
        // Broadcast a message on a given port.
        virtual void broadcast(int port, const std::string &msg) = 0;

        // Receive a message. For bound sockets, listens for incoming
        // packets on the port that was set in ctor; returns false if
        // no msg waiting, otherwise returns true and copies the msg
        // and address of sender into the two parameters. For unbound
        // sockets, always returns false.
        virtual bool receive(std::string &address, std::string &msg) = 0;
    };

}

#endif
